##########################################################################################################
#
# Calibre Auto-Updater
# --------------------
# 
# 1. Checks the current installed version of calibre.exe
# 2. Gets the "latest" version number from the web
# 3. If the current version is not the same as the latest version
#    a) Ensures that this is running in an elevated session
#    b) Shuts down calibre if it's currently running
#    c) Silently installs latest version directly from the web
#    d) Restarts calibre (with --disable-update-check seeing as you're now using this!)
# 
# Restrictions/Limitations
# ------------------------ 
# 
#  - As this stands it works for Calibre 64-bit (on a x64 Windows install) installed in the default location
#    a) If you are running Calibre x86 on a x64 Windows install
#       i)  Change both $CALIBRE_EXE to "C:\\Program Files (x86)\\Calibre2\\calibre.exe"
#       ii) Change $MSI_URL to "http://status.calibre-ebook.com/dist/win32"
#    b) If you are running Calibre on a 32-bit Windows install
#       i)  Change $MSI_URL to "http://status.calibre-ebook.com/dist/win32"
#    c) If you have installed Calibre somewhere other than the default location
#       i)  Change $CALIBRE_EXE to the appropriate location
#       ii) Then it might work (not sure what a "quiet" msiexec does when re-installing to a non-default locn)!
#
#  - If you want to run this as a scheduled task, make sure that it's running as elevated
#
#  - It will do the version detection without being elevated (then prompt you)
#
##########################################################################################################

$MSI_URL     = "http://status.calibre-ebook.com/dist/win64"
$CALIBRE_EXE = "C:\\Program Files\\Calibre2\\calibre.exe"
$stars       = "*".PadRight(80,"*")

$stars
"Calibre Auto-Updater"
$stars

$installedVersion = [System.Diagnostics.FileVersionInfo]::GetVersionInfo($CALIBRE_EXE).FileVersion

#
# This bit of code stops the HTTPS call to get the version from failing
#
if (-not ([System.Management.Automation.PSTypeName]'ServerCertificateValidationCallback').Type)
{
$certCallback = @"
    using System;
    using System.Net;
    using System.Net.Security;
    using System.Security.Cryptography.X509Certificates;
    public class ServerCertificateValidationCallback
    {
        public static void Ignore()
        {
            if(ServicePointManager.ServerCertificateValidationCallback ==null)
            {
                ServicePointManager.ServerCertificateValidationCallback +=
                    delegate
                    (
                        Object obj,
                        X509Certificate certificate,
                        X509Chain chain,
                        SslPolicyErrors errors
                    )
                    {
                        return true;
                    };
            }
        }
    }
"@
    Add-Type $certCallback
 }
[ServerCertificateValidationCallback]::Ignore()


# get the version from the web
$webVersion = ($urlCall = Invoke-WebRequest -URI https://code.calibre-ebook.com/latest).Content


$upgradeNeeded = !($installedVersion -eq $webVersion `
                 -or (($installedVersion.Length -gt $webVersion.Length) -and ($installedVersion.StartsWith($webVersion))) `
                 -or (($installedVersion.Length -lt $webVersion.Length) -and ($webVersion.StartsWith($installedVersion))) `
                  )

"Version Currently Installed : $installedVersion"
"Latest Version              : $webVersion"
"Upgrade Required            : $upgradeNeeded"
$stars

If ($upgradeNeeded) `
    {
     # Self-elevate the script if required
    if (-Not ([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] 'Administrator')) {
     if ([int](Get-CimInstance -Class Win32_OperatingSystem | Select-Object -ExpandProperty BuildNumber) -ge 6000) {
      $CommandLine = "-File `"" + $MyInvocation.MyCommand.Path + "`" " + $MyInvocation.UnboundArguments
      Start-Process -FilePath PowerShell.exe -Verb Runas -ArgumentList $CommandLine
      Exit
     }
    }
    "Shutting down calibre"
    Start-Process $CALIBRE_EXE -Wait -ArgumentList '--shutdown-running-calibre'
    "Installing new version      : $webVersion"
    Start-Process msiexec.exe -Wait -ArgumentList "/I $MSI_URL /qn"
    $installedVersion = [System.Diagnostics.FileVersionInfo]::GetVersionInfo($CALIBRE_EXE).FileVersion
    "Installed new version       : $installedVersion"
    "Restarting calibre"
    Start-Process $CALIBRE_EXE -ArgumentList '--no-update-check'
    $stars
    }